#!/usr/bin/perl

### ----------------------------------------------- ###
#  modular.pl
#  -----------
#
#  input:
#    messages.dtd
#    segments.dtd
#    fields.dtd
#
#
#  output: One DTD for every message structure
#          named in messages.dtd.
#
#  (last revised Dec 09, 1999)
### ----------------------------------------------- ###

open (messages, "messages.dtd");

$msg = "";
$i = 0;

#
# Preload all field declarations into associative array.
#
%fields = ();
open (flds, "fields.dtd");
while ($line = <flds>) {
  $item = $line; chop($item);
  $item =~ s/.*\%\s(\w+\.\w+).*/\1/;
  $line .= <flds>;
  $line .= <flds>;
  $line .= <flds>;
  $line .= <flds>;
  $line .= <flds>;
  $line .= <flds>;
  $line .= <flds>;
  $fields{$item} = $line;
  }
close (flds);

while (($line = <messages>) && ($i<5)) {
#  $i++;
  chop ($line);
  
  $msg = $line;
  $msg =~ s/.*\%\s(\w*)\..*/\1/;
  
  $msgfile = $msg.".dtd";
  open (msg, ">$msgfile");
  
  #
  # Add header to each file stating which message is being defined.
  # Add entity declaration for datatype DTD to start of each file.
  # Add first line of message declaration just read.
  #
  print msg <<EOF ;
<!--
HL7 V2.3.1
XML DTD for message $msg
-->

<!-- Datatype declarations -->
<!ENTITY % HL7V231-datatypes PUBLIC 
  "-//HL7//DTD HL7 V2.3.1 datatype definitions//EN" 
  "datatypes.dtd" >
%HL7V231-datatypes;

<!-- Message declarations -->
$line
EOF
  
  # if <!ENTITY, extract what is between double-quotes
  # turn it into a comma-seperated set.
  # foreach element in set
    # if there is a ".", exclude altogether
    # strip off "?", "*", "+"
    # add to list of segments
  
  @segmentlist = ();
  if ($line =~ /ENTITY/) {
    $line =~ s/.*\"(.*)\".*/\1/;
    $line =~ s/\(//g;
    $line =~ s/\)//g;
    @list = split(/,/,$line);
    foreach $item (@list) {
      $item =~ s/\?//g;
      $item =~ s/\*//g;
      $item =~ s/\+//g;
	  $item =~ s/\(//g;
	  $item =~ s/\)//g;
      if ($item =~ /\./) {} else {@segmentlist = (@segmentlist, $item)}
    }
  }
  
  #
  # Keep reading <messages> until you come to a blank line, copying each line into $msgfile.
  # 
  until ($line eq "") {
    $line = <messages>; 
    chop($line);
    print msg "$line\n";
    #
    # Extract segment names from the line read, and store them
    #
    if ($line =~ /ENTITY/) {
      $line =~ s/.*\"(.*)\".*/\1/;
	  $line =~ s/\(//g;
	  $line =~ s/\)//g;
      @list = split(/,/,$line);
      foreach $item (@list) {
        $item =~ s/\?//g;
        $item =~ s/\*//g;
        $item =~ s/\+//g;
        if ($item =~ /\./) {} else {@segmentlist = (@segmentlist, $item)}
      }
    }
  }
  
  #
  # Alphabetize list of segments and remove duplicates.
  #
  @segmentlist = sort(@segmentlist);
  $lastitem = "";
  @temp = ();
  foreach $item (@segmentlist) {
    if ($item ne $lastitem) {@temp = (@temp,$item)}
    $lastitem = $item;
  }
  @segmentlist = @temp;
 
# print msg "segmentlist: \"@segmentlist\"\n";
 
  open (segments, "segments.dtd");
  print msg "<!-- Segment declarations -->\n";
  $line = "";
  @fieldlist = ();
  foreach $seg (@segmentlist) {
    #
    # Find and print the first line containing the correct segment.
    #
    until ($line =~ /\<\!ENTITY \% $seg/) {$line = <segments>; chop($line)}
    print msg "$line\n";
    #
    # Add this line to $msgfile.
    #
    if ($line =~ /ENTITY/) {
      $line =~ s/.*\((.*)\).*/\1/;
      @list = split(/,/,$line);
      foreach $item (@list) {
        $item =~ s/\?//;
        $item =~ s/\*//;
        $item =~ s/\+//;
        if ($item =~ /LST/) {} else {@fieldlist = (@fieldlist, $item)}
      }
    }
    
    #
    # Then copy lines into $msgfile until you come to a blank line.
    #
    until ($line eq "") {
      $line = <segments>; 
      chop($line);
      print msg "$line\n";
      #
      # Extract field names from the line read, and store them
      #
      if ($line =~ /ENTITY/) {
        $line =~ s/.*\((.*)\).*/\1/;
        @list = split(/,/,$line);
        foreach $item (@list) {
          $item =~ s/\?//;
          $item =~ s/\*//;
          $item =~ s/\+//;
          if ($item =~ /LST/) {} else {@fieldlist = (@fieldlist, $item)}
        }
      }
    }
    
    #
    # Alphabetize list of fields and remove duplicates.
    #
    @fieldlist = sort(@fieldlist);
    $lastitem = "";
    @temp = ();
    foreach $item (@fieldlist) {
      if ($item ne $lastitem) {@temp = (@temp,$item)}
      $lastitem = $item;
    }
    @fieldlist = @temp;
  }
  close (segments);

# print msg "fieldlist: \"@fieldlist\"\n";

  #
  # Add field declarations to $msgfile.
  #
  print msg "<!-- Field declarations -->\n";
  foreach $item (@fieldlist) {print msg "$fields{$item}\n"}

  close (msg);
}
close (messages);

